--[======================================================[
					Lua Matrix Library
--]======================================================]

--[==[

Version:		0.9.1
Last Modified:	7/9/2012

Copyright 2012 Erik Madson

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

PM:
	http://www.omnimaga.org/ 
	Username: 3rik

Email:
	3rik.ommni@gmail.com

Forum Topics:
	Math - http://www.omnimaga.org/index.php?topic=13816
	Lua  - http://www.omnimaga.org/index.php?topic=13853


Thanks to Nick at omnimaga.org for offering suggestions on the math aspect of this library
Also, thanks to Levak and jimbauwens at omnimaga.org for helping me to organize my thoughts on how to organize the matrix.

==========================================================
					Documentation
==========================================================

This library adds a new matrix "type" to Lua. This library was intended to be compatible with
both standard Lua and TI-nSpire Lua. See http://education.ti.com/nspire/scripting-api 
or http://www.inspired-lua.org/ for more details on this version of Lua.

~~~~~~~~~~~~~~~~~~~~~~~~~~~
Global variables consumed:
~~~~~~~~~~~~~~~~~~~~~~~~~~~

matrix
_raw

~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Global functions redefined:
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

type
math.abs
math.acos
math.asin
math.atan
math.ceil
math.cos
math.cosh
math.deg
math.exp
math.floor
math.log
math.log10
math.max
math.min
math.rad
math.sin
math.sinh
math.tan
math.tanh

----------------------------------------------------------
					Syntax
----------------------------------------------------------
Matrices behave similar to tables (probably because they actually are tables).
However, they are more structured than tables, therefore more strict about inputs.

All matrix related functions are contained in the table called matrix (see "Functions" for documentation on each function).

To create a new matrix use

matrix.new(tbl)

where tbl is a table where all each key is an integer greater than zero (if a key isn't, it won't be in the matrix) and each value is another table.
This table's keys need to fit the same criteria and must contain all numbers (or strings that will be able to convert to numbers with the tonumber function).
If a value isn't a number, it will be stored as a zero, with the exception of true which will be stored as a 1.

For example:

matrix.new({{7, 5, 8}, {6, 9, 3}})

would be acceptable. As a note, all zeros are stored as nils to save space if a user is creating a large sparse matrix on the TI-nSpire. Empty tables are also stored this way.

This is also acceptable:

matrix.new({[10] = {[4] = 0}})

This is stored the same way as

matrix.new({{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}})


Assignment is similar to but a bit different from assignment for tables.
When assigning a matrix to a variable, it is assign by reference, like tables.

mat1 = matrix.new({{7, 5, 8}, {6, 9, 3}})
mat2 = mat1

mat1 and mat2 are now pointing to the same matrix. Any changes to mat1 will show in mat2 and vice versa.
To remedy this situation there is a function called matrix.copy (see below). So

mat3 = matrix.copy(mat1)

would create a copy of the matrix and pass that reference to mat2.

To assign a new value to an element in a matrix, do

mat1[1][3] = 4

The number that would have been 8 in the example above would now be 4 (mat1[1][3] == 4, mat2[1][3] == 4, mat3[1][3] == 8)

You can also assign a whole row at a time.

mat1[2] = {5} --same as {5, 0, 0}

Be aware that you cannot assign to values outside the dimensions of the matrix.

To access the dimensions of a matrix use mat1.dim[1] or mat1.dim.rows for the row dimension and mat1.dim[2] or mat1.dim.cols

Matrices make use of some operators as well

mat1 + mat2 --adds each corresponding element
mat1 - mat2
- mat1 --returns the opposite of each element
mat1 * mat2 --preforms matrix multiplication. (In this case an error will occur.)
mat1 == mat2 --bases equality on values not on reference

mat1 == matrix.copy(mat1) -- this will return true

Also, note that all the functions in the matrix library can be accessed through a matrix themselves.
This means that

matrix.copy(mat1) is the same as mat1.copy(mat1) and mat1:copy()

Also, 

type(mat1) == "matrix"

tostring(mat1) is also supported

This covers the basics. There are a few minor quirks (especially with rawget and rawset) but there shouldn't be anything major. If you have any questions, comments, suggestions, corrections, or concerns, my contact information is listed above. I recommend
posting questions via the forum so everyone can receive the benefit of question.

----------------------------------------------------------
					Functions
----------------------------------------------------------

~~~~~~~~~~~~~~~~~~~~~~
New Functions
~~~~~~~~~~~~~~~~~~~~~~

_raw.passerror
matrix.new
matrix.isequal
matrix.totable
matrix.copy
matrix.mpairs
matrix.add
matrix.sub
matrix.ebemult
matrix.ebediv
matrix.ebepow
matrix.neg
matrix.mult
matrix.trans
matrix.gen.zero
matrix.gen.identity
matrix.gen.random
matrix.submatrix
matrix.insert.row
matrix.insert.col
matrix.remove.row
matrix.remove.col


~~~~~~~~~~~~~~~~
_raw.type ( data )
~~~~~~~~~~~~~~~~

	Lua's standard type function

	mat1 = matrix.new({{7, 5, 8}, {6, 9, 3}}
	_raw.type(mat1) == "table"

	See http://www.lua.org/manual/5.1/manual.html#pdf-type for details

~~~~~~~~~~
_raw.math
~~~~~~~~~~

	Contains most of Lua's standard math functions
	See http://www.lua.org/manual/5.1/manual.html#5.6 for details

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_raw.passerror ( func [, errstr [, ... ] ] )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	_raw.passerror calls the function func(...).
	Any errors that occur within func will have the line number of the function that calls _raw.passerror.
	There is also an option to add an additional error message to the one generated by func.
	This is done by including a string in errstr. If errstr has a false value or "" then only the original error message will be printed.

	This does not stop a function from producing an error; it just provides a method to trace back to the programmer's code not the code in the matrix library.

	If no error occurs then the function returns all the values that the function would have.

	This is meant for use within a function that the programmer using this library (or any other library for that matter) didn't write.

	This could likely be accomplished using the debug library, but this library was written to be compatible with the TI-nSpire 
	which doesn't support the debug library.

~~~~~~~~~~~~~~~~~~~~~
redefined functions
~~~~~~~~~~~~~~~~~~~~~

	type, math.abs, math.acos, math.asin, math.atan, math.ceil, math.cos, math.cosh, math.deg,
	math.exp, math.floor, math.log, math.log10, math.max, math.min, math.rad, math.sin,
	math.sinh, math.tan, and math.tanh have been redefined to incorporate matrices.

	Each math function is evaluated element by element for matrices.

	For example:

	mat1 = matrix.new({{-4, 6, 7, 4, -7}})
	math.abs(mat1) == matrix.new({{4, 6, 7, 4, 7}})

~~~~~~~~~~~~~~~~~~~
matrix.new ( data )
~~~~~~~~~~~~~~~~~~~

	Creates a new matrix from a table. See syntax for more information.

	matrix.new({{-4, 6, 7, 4, -7}})

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.isequal ( mat1, mat2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Returns true if all the values in mat1 are equal to the corresponding value in mat2 and the dimensions of the matrices are the same.

	mat1:isequal(mat2) == (mat1 == mat2)

~~~~~~~~~~~~~~~~~~~~~~~
matrix.totable ( mat )
~~~~~~~~~~~~~~~~~~~~~~~

	Converts mat into a table. All nils are converted to zeros.

	mat1 = matrix.new({{-4, 6, 7}, [3] = {4, -7}})
	mat1:totable() produces {{-4, 6, 7}, {0, 0, 0}, {4, -7, 0}}

~~~~~~~~~~~~~~~~~~~~~~
matrix.mpairs ( mat )
~~~~~~~~~~~~~~~~~~~~~~

	Used like ipairs for a table. Returns the row, column, and value.

	mat1 = matrix.new({{7, 5, 8}, {6, 9, 3}})
	for row, col, val in mat1:mpairs() do
		print(row, col, val)
	end

	This will print
	1	1	7
	1	2	5
	1	3	8
	2	1	6
	2	2	9
	2	3	3

~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.add ( data1, data2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	data1, data2, or both must be matrices. If one is a matrix, the other must be a number or a matrix. Adds each element with the corresponding element.

	data1:add(data2) == data1 + data2

~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.sub ( data1, data2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	data1, data2, or both must be matrices. If one is a matrix, the other must be a number or a matrix. Subtracts each element with the corresponding element.

	data1:sub(data2) == data1 - data2

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.ebemult ( data1, data2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	element-by-element multiplication

	data1, data2, or both must be matrices. If one is a matrix, the other must be a number or a matrix. Multiplies each element with the corresponding element.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.ebediv ( data1, data2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	element-by-element division

	data1, data2, or both must be matrices. If one is a matrix, the other must be a number or a matrix. Divides each element with the corresponding element.
	data2 may not be a zero or a matrix containing a zero.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.ebepow ( data1, data2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	element-by-element exponentiation

	data1, data2, or both must be matrices. If one is a matrix, the other must be a number or a matrix. Raises each element to the power of the corresponding element.
	Normal rules regarding real exponents apply.

~~~~~~~~~~~~~~~~~~~
matrix.neg ( mat )
~~~~~~~~~~~~~~~~~~~

	Makes each element of mat opposite.

	mat1 = matrix.new({{7, 5, 8}, {6, 9, 3}})
	-mat1 == mat1:neg()

~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.mult ( mat1, mat2 )
~~~~~~~~~~~~~~~~~~~~~~~~~~~

	matrix multiplication.

	For information regarding this topic, see:
	http://en.wikipedia.org/wiki/Matrix_multiplication
	http://www.purplemath.com/modules/mtrxmult.htm
	http://mathworld.wolfram.com/MatrixMultiplication.html

	For the scalar product, use ebemult.

	mat1 * mat2 == mat1:mult(mat2)

~~~~~~~~~~~~~~~~~~~~~
matrix.trans ( mat )
~~~~~~~~~~~~~~~~~~~~~

	Swaps the rows with the columns

	For example:

	mat1 = matrix.new({{7, 5, 8}, {6, 9, 3}})
	print(mat1:trans())

	will print

	{	{7,	6},
		{5,	9},
		{8,	3}}

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.gen.zero (dim1 [, dim2])
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Generates a matrix where all the values are zero.
	Assumes dim1 x dim1 matrix unless dim2 is specified.

~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.gen.identity (dim)
~~~~~~~~~~~~~~~~~~~~~~~~~~

	Creates a dim x dim identity matrix

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.gen.random (dim1 [, dim2 [[, lower], upper]] )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Generates a random matrix. Assumes square unless dim2 is specified.
	lower and upper are arguments to math.random. See http://www.lua.org/manual/5.1/manual.html#pdf-math.random for more info.
	Note that dim2 is required for lower and upper to be accepted.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.submatrix ( mat, row_start, row_end, column_start, column_end )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Much like string.sub.

	Returns a matrix that contains the data between (inclusive) the specified rows and columns.

	Negative integers are accepted (number from the end rather than the start).

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.insert.row ( mat, [pos,] rowdata )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Returns a matrix with a row inserted. If pos isn't specified, the row will be added at the end.
	Note that this function returns a matrix rather than causing a side effect.

	mat1 = matrix.insert.row(mat1, {5, 6})

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.insert.col ( mat, [pos,] columndata )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Returns a matrix with a column inserted. If pos isn't specified, the row will be added at the end.
	Note that this function returns a matrix rather than causing a side effect.

	mat1 = matrix.insert.col(mat1, {5, 6})

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.remove.row ( mat [, pos] )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Returns a matrix with a row removed and a copy of the data in the row. If pos isn't specified, the row will be removed from the end.
	Note that this function returns a matrix rather than causing a side effect.

	mat1, oldrow = matrix.remove.row(mat1)

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix.remove.col ( mat [, pos] )
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	Returns a matrix with a column removed and a copy of the data in the column. If pos isn't specified, the column will be removed from the end.
	Note that this function returns a matrix rather than causing a side effect.

	mat1, oldcolumn = matrix.remove.row(mat1)


----------------------------------------------------------
					Code
----------------------------------------------------------

]==]


assert(_VERSION == "Lua 5.1", "This library was written for Lua 5.1. It may not work correctly with your version.\nVisit http://www.omnimaga.org/index.php?topic=13853 to check for updates.")

_raw = { --contains all the references to the original Lua functions (and passerror since I didn't want to clutter the global variables too much)
	type = type,
	math = { --note that this does not contain all the math functions
		abs = math.abs,
		acos = math.acos,
		asin = math.asin,
		atan = math.atan,
		ceil = math.ceil,
		cos = math.cos,
		cosh = math.cosh,
		deg = math.deg,
		exp = math.exp,
		floor = math.floor,
		log = math.log,
		log10 = math.log10,
		max = math.max,
		min = math.min,
		rad = math.rad,
		sin = math.sin,
		sinh = math.sinh,
		tan = math.tan,
		tanh = math.tanh
	},
	passerror = function(func, errstr, ...)
		errstr = errstr or "" --if there is no value in errstr default to the empty string
		if type(func) ~= "function" then
			error("_raw.passerror: attempt to call a " .. type(func) .. " value\n", 2)
		end
		errstr = tostring(errstr)
		local returns = {pcall(func, ...)} --calls the function in protected mode
		if not returns[1] then --if an error occured
			errstr = errstr .. (errstr == "" and "" or "\n\t") .. string.sub(returns[2], (select(2, string.find(returns[2], "^%[string \"%-%-%[========================================%.%.%.\"%]:%d+: ")) or 0) + 1) --prefix the error with errstr
			error(errstr, 3) --send the error up the call stack
		end
		return  unpack(returns, 2, table.maxn(returns)) --if the function excecuted correctly, return all the results that it would've
	end
}

function type(val)
	if getmetatable(val) == "matrix" then --the signal that a table is actually a matrix
		return "matrix"
	else --if val isn't a matrix then call type() normally
		return _raw.type(val)
	end
end

function math.abs(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.abs: unexpected matrix error", data) do --loops through the matrix
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.abs, "math.abs: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.abs: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.abs, "math.abs:", data) --prevents the error from printing this line number instead of the line of the actuall error
	end
end

function math.acos(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.acos: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.acos, "math.acos: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.acos: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.acos, "math.acos:", data)
	end
end

function math.asin(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.asin: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.asin, "math.asin: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.asin: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.asin, "math.asin:", data)
	end
end

function math.atan(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.atan: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.atan, "math.atan: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.atan: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.atan, "math.atan:", data)
	end
end

function math.ceil(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.ceil: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.ceil, "math.ceil: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.ceil: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.ceil, "math.ceil:", data)
	end
end

function math.cos(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.cos: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.cos, "math.cos: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.cos: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.cos, "math.cos:", data)
	end
end

function math.cosh(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.cosh: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.cosh, "math.cosh: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.cosh: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.cosh, "math.cosh:", data)
	end
end

function math.deg(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.deg: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.deg, "math.deg: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.deg: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.deg, "math.deg:", data)
	end
end

function math.exp(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.exp: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.exp, "math.exp: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.exp: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.exp, "math.exp:", data)
	end
end

function math.floor(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.floor: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.floor, "math.floor: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.floor: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.floor, "math.floor:", data)
	end
end

function math.log(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.log: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.log, "math.log: bad matrix", val == 0 and -1 or val)
		end
		return _raw.passerror(matrix.new, "math.log: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.log, "math.log:", data)
	end
end

function math.log10(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.log10: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.log10, "math.log10: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.log10: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.log10, "math.log10:", data)
	end
end

function math.max(data, ...)
	if type(data) == "matrix" then
		local maxi = 0
		for row, col, val in _raw.passerror(data.mpairs, "math.max: unexpected matrix error", data) do
			maxi = maxi > val and maxi or val --finds the maximum value in the matrix
		end
		return maxi
	else
		return _raw.passerror(_raw.math.max, "math.max:", data, ...)
	end
end

function math.min(data, ...)
	if type(data) == "matrix" then
		local mini = 0
		for row, col, val in _raw.passerror(data.mpairs, "math.min: unexpected matrix error", data) do
			mini = mini < val and mini or val --finds the minimum value in the matrix
		end
		return mini
	else
		return _raw.passerror(_raw.math.min, "math.min:", data, ...)
	end
end

function math.rad(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.rad: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.rad, "math.rad: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.rad: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.rad, "math.rad:", data)
	end
end

function math.sin(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.sin: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.sin, "math.sin: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.sin: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.sin, "math.sin:", data)
	end
end

function math.sinh(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.sinh: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.sinh, "math.sinh: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.sinh: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.sinh, "math.sinh:", data)
	end
end

function math.tan(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.tan: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.tan, "math.tan: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.tan: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.tan, "math.tan:", data)
	end
end

function math.tanh(data)
	if type(data) == "matrix" then
		local newmat = {}
		for row, col, val in _raw.passerror(data.mpairs, "math.tanh: unexpected matrix error", data) do
			newmat[row] = newmat[row] or {}
			newmat[row][col] = _raw.passerror(_raw.math.tanh, "math.tanh: bad matrix", val)
		end
		return _raw.passerror(matrix.new, "math.tanh: invalid matrix", newmat)
	else
		return _raw.passerror(_raw.math.tanh, "math.tanh:", data)
	end
end

matrix = {

	new = function(data)
		if type(data) ~= "table" then
			error("matrix.new: argument 1 must be a table", 2)
		end
		local mat, raw, cols, rows = {}, {}, 0, table.maxn(data)
		if rows <= 0 then
			error("matrix.new: matrix must contain at least one row", 2)
		end
		if math.floor(rows) ~= math.ceil(rows) then
			error("matrix.new: matrix must contain only integer numbered rows", 2)
		end
		local raw_i_meta = {
			__index = function()
				return 0
			end
		}
		for i = 1, rows do
			if type(data[i]) ~= "table" and type(data[i]) ~= "nil" then
				error("matrix.new: all non-nil rows must be tables", 2)
			end
			raw[i] = {}
			setmetatable(raw[i], raw_i_meta)
			if data[i] then
				local t = table.maxn(data[i])
				cols = t <= cols and cols or t
			end
		end
		if cols <= 0 then
			error("matrix.new: matrix must contain at least one column")
		end
		if math.floor(cols) ~= math.ceil(cols) then
			error("matrix.new: matrix must contain only integer numbered columns", 2)
		end
		for i = 1, rows do
			if data[i] ~= nil then
				local maxi, mini = 0, 0
				for j = 1, cols do
					if data[i][j] ~= nil and tonumber(data[i][j]) ~= 0 then
							local testvar = data[i][j] == true and 1 or tonumber(data[i][j])
							if tostring(math.abs(testvar or 0)) == tostring(math.huge) or tostring(math.abs(testvar or 0)) == tostring(math.abs(math.acos(2))) then
								error("undefined numbers are invalid in matrices", 2)
							end
							rawset(raw[i], j, testvar)

						maxi = maxi >= raw[i][j] and maxi or raw[i][j]
						mini = mini <= raw[i][j] and mini or raw[i][j]
					end
				end
				if mini == 0 and maxi == 0 then
					raw[i] = nil
				end
			end
		end
		setmetatable(raw, {
			__index = function(tbl, key)
				key, temp = _raw.passerror(tonumber, "index failed", key), {}
				if not key then
					error("index failed", 2)
				end
				if math.floor(key) ~= math.ceil(key) or key <= 0 or key > rows then
					error("index failed", 2)
				end
				setmetatable(temp, {
					__index = function()
						return 0
					end,
					__newindex = function(tbl, key2, val)
						key2 = _raw.passerror(tonumber, "index failed", key)
						if not key2 then
							error("index failed", 2)
						end
						if math.floor(key2) ~= math.ceil(key2) or key2 <= 0 or key2 > cols then
							error("index failed", 2)
						end
						rawset(raw, key, {})
						setmetatable(raw[key], {
							__index = function()
								return 0
							end
						})
						local testvar = val == true and 1 or tonumber(val)
						if tostring(math.abs(testvar or 0)) == tostring(math.huge) or tostring(math.abs(testvar or 0)) == tostring(math.abs(math.acos(2))) then
							error("undefined numbers are invalid in matrices", 2)
						end
						rawset(raw[key], key2, testvar ~= 0 and testvar or nil)
						if table.maxn(raw[key]) == 0 then
							rawset(raw, key, nil)
						end
					end
				})
				return temp
			end,
			__newindex = function(tbl, key, val)
				if type(val) == "table" then
					local maxi, mini = 0, 0
					key = _raw.passerror(tonumber, "index failed", key)
					if not key then
						error("index failed", 2)
					end
					if math.floor(key) ~= math.ceil(key) or key <= 0 or key > rows then
						error("index failed", 2)
					end
					rawset(raw, key, {})
					setmetatable(raw[key], {
						__index = function()
							return 0
						end
					})
					for j = 1, cols do
						if val[j] ~= nil and tonumber(val[j]) ~= 0 then
							local testvar = val[j] == true and 1 or tonumber(val[j])
							if tostring(math.abs(testvar or 0)) == tostring(math.huge) or tostring(math.abs(testvar or 0)) == tostring(math.abs(math.acos(2))) then
								error("undefined numbers are invalid in matrices", 2)
							end
							rawset(raw[key], j, testvar)
							maxi = maxi >= raw[key][j] and maxi or raw[key][j]
							mini = mini <= raw[key][j] and mini or raw[key][j]
						end
					end
					if mini == 0 and maxi == 0 then
						rawset(raw, key, nil)
					end
				elseif type(val) == "nil" then
					rawset(raw, key, nil)
				end
			end				
		})
		local meta = {
			__index = function(tbl, key)
				if key == "dim" then
					return {rows, cols, rows = rows, cols = cols}
				end
				if matrix[key] and key ~= tonumber(key) then
					return matrix[key]
				end
				key = _raw.passerror(tonumber, "index failed", key)
				if not key then
					error("index failed", 2)
				end
				if math.floor(key) ~= math.ceil(key) or key <= 0 or key > rows then
					error("index failed", 2)
				end
				local rowcopy = {}
				setmetatable(rowcopy, {
					__index = raw[key],
					__newindex = function(tbl, key2, val)
						key2 = _raw.passerror(tonumber, "index failed", key2)
						if not key2 then
							error("index failed", 2)
						end
						if math.floor(key2) ~= math.ceil(key2) or key2 <= 0 or key2 > cols then
							error("index failed", 2)
						end
						if not raw[key] then
							raw[key] = {}
							setmetatable(raw[key], {
								__index = function()
									return 0
								end
							})
						end
						local testvar = val == true and 1 or tonumber(val)
						if tostring(math.abs(testvar or 0)) == tostring(math.huge) or tostring(math.abs(testvar or 0)) == tostring(math.abs(math.acos(2))) then
							error("undefined numbers are invalid in matrices", 2)
						end
						rawset(raw[key], key2, testvar ~= 0 and testvar or nil)
						if table.maxn(raw[key]) == 0 then
							raw[key] = nil
						end
					end,
					__metatable = false
				})
				return rowcopy
			end,
			__newindex = function(tbl, key, val)
				key = _raw.passerror(tonumber, "index failed", key)
				if not key then
					error("index failed", 2)
				end
				if math.floor(key) ~= math.ceil(key) or key <= 0 or key > rows then
					error("index failed", 2)
				end
				if type(val) == "table" then
					local maxi, mini = 0, 0
					raw[key] = {}
					setmetatable(raw[key], {
						__index = function()
							return 0
						end
					})
					for j = 1, cols do
						if val[j] ~= nil and tonumber(val[j]) ~= 0 then
							local testvar = val[j] == true and 1 or tonumber(val[j])
							if tostring(math.abs(testvar or 0)) == tostring(math.huge) or tostring(math.abs(testvar or 0)) == tostring(math.abs(math.acos(2))) then
								error("undefined numbers are invalid in matrices", 2)
							end
							rawset(raw[key], j, testvar)
							maxi = maxi >= raw[key][j] and maxi or raw[key][j]
							mini = mini <= raw[key][j] and mini or raw[key][j]
						end
					end
					if mini == 0 and maxi == 0 then
						raw[key] = nil
					end
				elseif type(val) == "nil" then
					raw[key] = nil
				end
			end,
			__metatable = "matrix",
			__tostring = function(tbl)
				local str = "{"
				for i = 1, tbl.dim[1] do
					str = str .. "\t{"
					for j = 1, tbl.dim[2] do
						str = str .. tostring(tbl[i][j]) .. ",\t"
					end
					str = str:sub(1, -3) .. "},\n"
				end
				str = str:sub(1, -3) .. "}"
				return str
			end,
			__eq = matrix.isequal,
			__add = matrix.add,
			__sub = matrix.sub,
			__mul = matrix.mult,
			__unm = matrix.neg
		}
		setmetatable(mat, meta)
		return mat
	end,

	isequal = function(mat1, mat2)
		if type(mat1) == "matrix" and type(mat2) == "matrix" and mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2] then
			for i = 1, mat1.dim[1] do
				for j = 1, mat1.dim[2] do
					if mat1[i][j] ~= mat2[i][j] then
						return false
					end
				end
			end
			return true
		end
		return rawequal(mat1, mat2)
	end,

	totable = function(mat)
		if type(mat) ~= "matrix" then
			error("matrix.totable: argument 1 must be a matrix", 2)
		end
		local tbl = {}
		for i = 1, mat.dim[1] do
			tbl[i] = {}
			for j = 1, mat.dim[2] do
				tbl[i][j] = mat[i][j]
			end
		end
	return tbl
	end,

	copy = function(mat)
		return _raw.passerror(matrix.new, "matrix.copy:", _raw.passerror(mat.totable, "matrix.copy:", mat))
	end,

	mpairs = function(mat)
		if type(mat) ~= "matrix" then 
			error("matrix.mpairs: argument 1 must be a matrix", 2)
		end
		local i, j, m, n = 1, 0, mat.dim[1], mat.dim[2]
		return function()
			j = j + 1
			if j > n then
				j = 1
				i = i + 1
				if i > m then
					return
				end
			end
			return i, j, mat[i][j]
		end
	end,

	add = function(mat1, mat2)
		local newmat = {}
		if type(mat1) == "matrix" and type(mat2) == "matrix" then
			if not (mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2]) then
				error("matrix.add: dimention mismatch", 2)
			end
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val + mat2[row][col]
			end
			return _raw.passerror(matrix.new, "matrix.add:", newmat)
		elseif type(mat1) == "matrix" and type(tonumber(mat2)) == "number" then
			mat2 = tonumber(mat2)
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val + mat2
			end
			return _raw.passerror(matrix.new, "matrix.add:", newmat)
		elseif type(mat2) == "matrix" and type(tonumber(mat1)) == "number" then
			mat1 = tonumber(mat1)
			for row, col, val in mat2:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = mat1 + val
			end
			return _raw.passerror(matrix.new, "matrix.add:", newmat)
		end
	end,

	sub = function(mat1, mat2)
		local newmat = {}
		if type(mat1) == "matrix" and type(mat2) == "matrix" then
			if not (mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2]) then
				error("matrix.sub: dimention mismatch", 2)
			end
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val - mat2[row][col]
			end
			return _raw.passerror(matrix.new, "matrix.sub:", newmat)
		elseif type(mat1) == "matrix" and type(tonumber(mat2)) == "number" then
			mat2 = tonumber(mat2)
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val - mat2
			end
			return _raw.passerror(matrix.new, "matrix.sub:", newmat)
		elseif type(mat2) == "matrix" and type(tonumber(mat1)) == "number" then
			mat1 = tonumber(mat1)
			for row, col, val in mat2:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = mat1 - val
			end
			return _raw.passerror(matrix.new, "matrix.sub:", newmat)
		end
	end,

	ebemult = function(mat1, mat2)
		local newmat = {}
		if type(mat1) == "matrix" and type(mat2) == "matrix" then
			if not (mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2]) then
				error("matrix.ebemult: dimention mismatch", 2)
			end
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val * mat2[row][col]
			end
			return _raw.passerror(matrix.new, "matrix.ebemult:", newmat)
		elseif type(mat1) == "matrix" and type(tonumber(mat2)) == "number" then
			mat2 = tonumber(mat2)
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = val * mat2
			end
			return _raw.passerror(matrix.new, "matrix.ebemult:", newmat)
		elseif type(mat2) == "matrix" and type(tonumber(mat1)) == "number" then
			mat1 = tonumber(mat1)
			for row, col, val in mat2:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				newmat[row][col] = mat1 * val
			end
			return _raw.passerror(matrix.new, "matrix.ebemult:", newmat)
		end
	end,

	ebediv = function(mat1, mat2)
		local newmat = {}
		if type(mat1) == "matrix" and type(mat2) == "matrix" then
			if not (mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2]) then
				error("matrix.ebediv: dimention mismatch", 2)
			end
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if mat2[row][col] == 0 then
					error("matrix.ebediv: divide by zero", 2)
				end
				newmat[row][col] = val / mat2[row][col]
			end
			return _raw.passerror(matrix.new, "matrix.ebediv:", newmat)
		elseif type(mat1) == "matrix" and type(tonumber(mat2)) == "number" then
			mat2 = tonumber(mat2)
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if mat2 == 0 then
					error("matrix.ebediv: divide by zero", 2)
				end
				newmat[row][col] = val / mat2
			end
			return _raw.passerror(matrix.new, "matrix.ebediv:", newmat)
		elseif type(mat2) == "matrix" and type(tonumber(mat1)) == "number" then
			mat1 = tonumber(mat1)
			for row, col, val in mat2:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if val == 0 then
					error("matrix.ebediv: divide by zero", 2)
				end
				newmat[row][col] = mat1 / val
			end
			return _raw.passerror(matrix.new, "matrix.ebediv:", newmat)
		end
	end,

	ebepow = function(mat1, mat2)
		local newmat = {}
		if type(mat1) == "matrix" and type(mat2) == "matrix" then
			if not (mat1.dim[1] == mat2.dim[1] and mat1.dim[2] == mat2.dim[2]) then
				error("matrix.ebepow: dimention mismatch", 2)
			end
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if (val == 0 and - math.abs(mat2[row][col]) == mat2[row][col]) or (val < 0 and math.floor(mat2[row][col]) ~= math.ceil(mat2[row][col])) then
					error("matrix.ebepow: exponent error", 2)
				end
				newmat[row][col] = val ^ mat2[row][col]
			end
			return _raw.passerror(matrix.new, "matrix.ebepow:", newmat)
		elseif type(mat1) == "matrix" and type(tonumber(mat2)) == "number" then
			mat2 = tonumber(mat2)
			for row, col, val in mat1:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if (val == 0 and - math.abs(mat2) == mat2) or (val < 0 and math.floor(mat2) ~= math.ceil(mat2)) then
					error("matrix.ebepow: exponent error", 2)
				end
				newmat[row][col] = val ^ mat2
			end
			return _raw.passerror(matrix.new, "matrix.ebepow:", newmat)
		elseif type(mat2) == "matrix" and type(tonumber(mat1)) == "number" then
			mat1 = tonumber(mat1)
			for row, col, val in mat2:mpairs() do
				if col == 1 then
					newmat[row] = {}
				end
				if (mat1 == 0 and - math.abs(val) == val) or (mat1 < 0 and math.floor(val) ~= math.ceil(val)) then
					error("matrix.ebepow: exponent error", 2)
				end
				newmat[row][col] = mat1 ^ val
			end
			return _raw.passerror(matrix.new, "matrix.ebepow:", newmat)
		end
	end,

	neg = function(mat)
		if type(mat) ~= "matrix" then
			error("matrix.neg: argument 1 must be a matrix", 2)
		end
		return _raw.passerror(matrix.ebemult, "matrix.neg:", mat, -1)
	end,

	mult = function(mat1, mat2)
		if type(mat1) ~= "matrix" or type(mat2) ~= "matrix" then
			error("matrix.mult: arguments  must be a matrices", 2)
		elseif mat2.dim[1] ~= mat1.dim[2] then
			error("matrix.mult: dimention mismatch", 2)
		end
		local newmat = {}
		for i = 1, mat1.dim[1] do
			newmat[i] = {}
			for j = 1, mat2.dim[2] do
				local val = 0
				for k = 1, mat1.dim[2] do
					val = mat1[i][k] * mat2[k][j] + val
				end
				newmat[i][j] = val
			end
		end
		return _raw.passerror(matrix.new, "matrix.mult:", newmat)
	end,

	trans = function(mat)
		if type(mat) ~= "matrix" then
			error("matrix.trans: argument 1 must be a matrix", 2)
		end
		local newmat = {}
		for row, col, val in mat:mpairs() do
			if row == 1 then
				newmat[col] = {}
			end
			newmat[col][row] = mat[row][col]
		end
		return _raw.passerror(matrix.new, "matrix.trans:", newmat)
	end,

	gen = {

		zero = function(dim1, dim2)
			dim2 = dim2 or dim1
			dim1, dim2 = tonumber(dim1), tonumber(dim2)
			if not (dim1 and dim2) then
				error("matrix.gen.zero: aruguments must be numbers", 2)
			end
			return _raw.passerror(matrix.new, "matrix.gen.zero:", {[dim1] = {[dim2] = 0}})
		end,

		identity = function(dim)
			dim = tonumber(dim)
			if not dim then
				error("matrix.gen.identity: aruguments must be numbers", 2)
			end
			local newmat = {}
			for i = 1, dim do
				newmat[i] = {}
				newmat[i][i] = 1
			end
			return _raw.passerror(matrix.new, "matrix.gen.identity:", newmat)
		end,

		random = function(dim1, dim2, arg1, arg2)
			dim1, dim2, arg1, arg2 = tonumber(dim1), tonumber(dim2) or tonumber(dim1), tonumber(arg1), tonumber(arg2)
			if not (dim1 and dim2) then
				error("matrix.gen.random: aruguments must be numbers", 2)
			end
			local newmat = {}
			for i = 1, dim1 do
				newmat[i] = {}
				for j = 1, dim2 do
					newmat[i][j] = math.random(unpack({arg1, arg2}))
				end
			end
			return _raw.passerror(matrix.new, "matrix.gen.random:", newmat)
		end
	},

	submatrix = function(mat, rstart, rend, cstart, cend)
		if type(mat) ~= "matrix" then
			error("matrix.submatrix: argument 1 must be a matrix", 2)
		end
		rstart, cstart, rend, cend = tonumber(rstart), tonumber(cstart), tonumber(rend), tonumber(cend)
		if not (rstart and cstart and rend and cend) then
			error("matrix.submatrix: aruguments must be numbers", 2)
		end
		rend, cend, rstart, cstart = rend < 0 and mat.dim[1] + rend + 1 or rend, cend < 0 and mat.dim[2] + cend + 1 or cend, rstart < 0 and mat.dim[1] + rstart + 1 or rstart, cstart < 0 and mat.dim[2] + cstart + 1 or cstart
		if not(1 <= rstart and rstart <= rend and rend <= mat.dim[1] and 1 <= cstart and cstart <= cend and cend <= mat.dim[2]) then
			error("submatrix outside dimentions of matrix", 2)
		end
		local newmat = {}
		for i = 1, rend - rstart + 1 do
			newmat[i] = {}
			for j = 1, cend - cstart + 1 do
				newmat[i][j] = mat[rstart + i - 1][cstart + j - 1]
			end
		end
		return _raw.passerror(matrix.new, "matrix.submatrix", newmat)
	end,

	insert = {

		row = function(mat, pos, row)
			if type(mat) ~= "matrix" then
				error("matrix.insert.row: argument 1 must be a matrix", 2)
			end
			local tbl = mat:totable()
			if not row then
				pos, row = row, pos
			else
				pos = tonumber(pos) or mat.dim[1]
				if pos < 1 or math.ceil(pos) ~= math.floor(pos) or pos > mat.dim[1] + 1 then
					error("matrix.insert.row: insertion posistion out of range", 2)
				end
			end
			for i = mat.dim[1], pos or mat.dim[1] + 1, -1 do
				tbl[i + 1] = tbl[i]
			end
			tbl[pos or mat.dim[1] + 1] = row
			return _raw.passerror(matrix.submatrix, "matrix.insert.row:", _raw.passerror(matrix.new, "matrix.insert.row:", tbl), 1, mat.dim[1] + 1, 1, mat.dim[2])
		end,

		col = function(mat, pos, col)
			return _raw.passerror(matrix.trans, "matrix.insert.col:", _raw.passerror(matrix.insert.row, "matrix.insert.col:", _raw.passerror(matrix.trans, "matrix.insert.col:", mat), pos, col))
		end
	},

	remove = {

		row = function(mat, pos)
			if type(mat) ~= "matrix" then
				error("matrix.remove.row: argument 1 must be a matrix", 2)
			end
			local tbl, rettbl, pos = mat:totable(), {}, tonumber(pos) or mat.dim[1]
			if pos < 1 or math.ceil(pos) ~= math.floor(pos) or mat.dim[1] == 1 or pos > mat.dim[1] then
				error("matrix.remove.row: removal position is out of range", 2)
			end
			for j = 1, mat.dim[1] do
				rettbl[j] = mat[pos][j]
			end
			for i = pos or mat.dim[1], mat.dim[1] do
				tbl[i] = tbl[i + 1]
			end
			return _raw.passerror(matrix.submatrix, "matrix.remove.row:", _raw.passerror(matrix.new, "matrix.remove.row:", tbl), 1, mat.dim[1] - 1, 1, mat.dim[2]), rettbl
		end,

		col = function(mat, pos)
			mat, rettbl = _raw.passerror(matrix.remove.row, "matrix.remove.col:", _raw.passerror(matrix.trans, "matrix.remove.col:", mat), pos)
			return _raw.passerror(matrix.trans, "matrix.remove.col:", mat), rettbl
		end
	},

--[[
	det = function()
	end,

	pinv = function()
	end,

	lu = function()
	end,

	pow = function()
	end,

	qr = function()
	end,

	ref = function()
	end,

	rref = function()
	end,
--]]
}